/*
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2022 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.01 of the PHP license,      |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | https://www.php.net/license/3_01.txt                                 |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Authors: Santiago Lizardo <santiagolizardo@php.net>                  |
  |          Remi Collet <remi@php.net>                                  |
  +----------------------------------------------------------------------+
*/

#include "keyboard.h"
#include "rect.h"
#include "window.h"

/* {{{ proto SDL_Window SDL_GetKeyboardFocus(void)

 *  \brief Get the window which currently has keyboard focus.
extern DECLSPEC SDL_Window * SDLCALL SDL_GetKeyboardFocus(void);
 */
PHP_FUNCTION(SDL_GetKeyboardFocus)
{
	if (zend_parse_parameters_none() == FAILURE)
	{
		return;
	}
	sdl_window_to_zval(SDL_GetKeyboardFocus(), return_value, SDL_DONTFREE);
}
/* }}} */

/* {{{ proto array SDL_GetKeyboardState([int &numkeys])

 *  \brief Get a snapshot of the current state of the keyboard.
 *
 *  \param numkeys if non-NULL, receives the length of the returned array.
 *
 *  \return An array of key states. Indexes into this array are obtained by using ::SDL_Scancode values.
 *
 *  \b Example:
 *  \code
 *  const Uint8 *state = SDL_GetKeyboardState(NULL);
 *  if ( state[SDL_SCANCODE_RETURN] )   {
 *      printf("<RETURN> is pressed.\n");
 *  }
 *  \endcode
 extern DECLSPEC const Uint8 *SDLCALL SDL_GetKeyboardState(int *numkeys);
 */
PHP_FUNCTION(SDL_GetKeyboardState)
{
	zval *z_numkeys = NULL;
	int i, numkeys;
	const Uint8 *state;

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "|z", &z_numkeys))
	{
		return;
	}
	state = SDL_GetKeyboardState(&numkeys);
	array_init(return_value);
	for (i = 0; i < numkeys; i++)
	{
		add_next_index_long(return_value, state[i]);
	}

	if (z_numkeys)
	{
		ZEND_TRY_ASSIGN_REF_LONG(z_numkeys, numkeys);
	}
}
/* }}} */

/* {{{ proto int SDL_GetModState(void)

 *  \brief Get the current key modifier state for the keyboard.
 extern DECLSPEC SDL_Keymod SDLCALL SDL_GetModState(void);
 */
PHP_FUNCTION(SDL_GetModState)
{
	if (zend_parse_parameters_none() == FAILURE)
	{
		return;
	}
	RETVAL_LONG(SDL_GetModState());
}
/* }}} */

/* {{{ proto void SDL_SetModState(int modstate)

 *  \brief Set the current key modifier state for the keyboard.
 *
 *  \note This does not change the keyboard state, only the key modifier flags.
 extern DECLSPEC void SDLCALL SDL_SetModState(SDL_Keymod modstate);
 */
PHP_FUNCTION(SDL_SetModState)
{
	zend_long modstate;

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "l", &modstate))
	{
		return;
	}
	SDL_SetModState(modstate);
}
/* }}} */

/* {{{ proto int SDL_GetKeyFromScancode(int scancode)

 *  \brief Get the key code corresponding to the given scancode according
 *         to the current keyboard layout.
 *
 *  See ::SDL_Keycode for details.
 *
 *  \sa SDL_GetKeyName()
 extern DECLSPEC SDL_Keycode SDLCALL SDL_GetKeyFromScancode(SDL_Scancode scancode);
 */
PHP_FUNCTION(SDL_GetKeyFromScancode)
{
	zend_long scancode;

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "l", &scancode))
	{
		return;
	}
	RETVAL_LONG(SDL_GetKeyFromScancode(scancode));
}
/* }}} */

/* {{{ proto int SDL_GetScancodeFromKey(int key)

 *  \brief Get the scancode corresponding to the given key code according to the
 *         current keyboard layout.
 *
 *  See ::SDL_Scancode for details.
 *
 *  \sa SDL_GetScancodeName()
 extern DECLSPEC SDL_Scancode SDLCALL SDL_GetScancodeFromKey(SDL_Keycode key);
 */
PHP_FUNCTION(SDL_GetScancodeFromKey)
{
	zend_long key;

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "l", &key))
	{
		return;
	}
	RETVAL_LONG(SDL_GetScancodeFromKey(key));
}
/* }}} */

/* {{{ proto string SDL_GetScancodeName(int scancode)

 *  \brief Get a human-readable name for a scancode.
 *
 *  \return A pointer to the name for the scancode.
 *          If the scancode doesn't have a name, this function returns
 *          an empty string ("").
 *
 *  \sa SDL_Scancode
 extern DECLSPEC const char *SDLCALL SDL_GetScancodeName(SDL_Scancode scancode);
 */
PHP_FUNCTION(SDL_GetScancodeName)
{
	zend_long scancode;

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "l", &scancode))
	{
		return;
	}
	RETVAL_STRING(SDL_GetScancodeName(scancode));
}
/* }}} */

/* {{{ proto int SDL_GetScancodeFromName(string name)

 *  \brief Get a scancode from a human-readable name
 *
 *  \return scancode, or SDL_SCANCODE_UNKNOWN if the name wasn't recognized
 *
 *  \sa SDL_Scancode
 extern DECLSPEC SDL_Scancode SDLCALL SDL_GetScancodeFromName(const char *name);
 */
PHP_FUNCTION(SDL_GetScancodeFromName)
{
	char *name;
	size_t name_len;

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "s", &name, &name_len))
	{
		return;
	}
	RETVAL_LONG(SDL_GetScancodeFromName(name));
}
/* }}} */

/* {{{ proto string SDL_GetKeyName(int key)

 *  \brief Get a human-readable name for a key.
 *
 *  \return A pointer to a UTF-8 string that stays valid at least until the next
 *          call to this function. If you need it around any longer, you must
 *          copy it.  If the key doesn't have a name, this function returns an
 *          empty string ("").
 *
 *  \sa SDL_Key
 extern DECLSPEC const char *SDLCALL SDL_GetKeyName(SDL_Keycode key);
 */
PHP_FUNCTION(SDL_GetKeyName)
{
	zend_long key;

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "l", &key))
	{
		return;
	}
	RETVAL_STRING(SDL_GetKeyName(key));
}
/* }}} */

/* {{{ proto int SDL_GetKeyFromName(string name)

 *  \brief Get a key code from a human-readable name
 *
 *  \return key code, or SDLK_UNKNOWN if the name wasn't recognized
 *
 *  \sa SDL_Keycode
 extern DECLSPEC SDL_Keycode SDLCALL SDL_GetKeyFromName(const char *name);
 */
PHP_FUNCTION(SDL_GetKeyFromName)
{
	char *name;
	size_t name_len;

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "s", &name, &name_len))
	{
		return;
	}
	RETVAL_LONG(SDL_GetKeyFromName(name));
}
/* }}} */

/* {{{ proto void SDL_StartTextInput(void)

 *  \brief Start accepting Unicode text input events.
 *         This function will show the on-screen keyboard if supported.
 *
 *  \sa SDL_StopTextInput()
 *  \sa SDL_SetTextInputRect()
 *  \sa SDL_HasScreenKeyboardSupport()
 extern DECLSPEC void SDLCALL SDL_StartTextInput(void);
 */
PHP_FUNCTION(SDL_StartTextInput)
{
	if (zend_parse_parameters_none() == FAILURE)
	{
		return;
	}
	SDL_StartTextInput();
}
/* }}} */

/* {{{ proto bool SDL_IsTextInputActive(void)

 *  \brief Return whether or not Unicode text input events are enabled.
 *
 *  \sa SDL_StartTextInput()
 *  \sa SDL_StopTextInput()
 extern DECLSPEC SDL_bool SDLCALL SDL_IsTextInputActive(void);
 */
PHP_FUNCTION(SDL_IsTextInputActive)
{
	if (zend_parse_parameters_none() == FAILURE)
	{
		return;
	}
	RETVAL_BOOL(SDL_IsTextInputActive());
}
/* }}} */

/* {{{ proto void SDL_StopTextInput(void)

 *  \brief Stop receiving any text input events.
 *         This function will hide the on-screen keyboard if supported.
 *
 *  \sa SDL_StartTextInput()
 *  \sa SDL_HasScreenKeyboardSupport()
 extern DECLSPEC void SDLCALL SDL_StopTextInput(void);
 */
PHP_FUNCTION(SDL_StopTextInput)
{
	if (zend_parse_parameters_none() == FAILURE)
	{
		return;
	}
	SDL_StopTextInput();
}
/* }}} */

/* {{{ proto void SDL_SetTextInputRect(SDL_Rect rect)

 *  \brief Set the rectangle used to type Unicode text inputs.
 *         This is used as a hint for IME and on-screen keyboard placement.
 *
 *  \sa SDL_StartTextInput()
 extern DECLSPEC void SDLCALL SDL_SetTextInputRect(SDL_Rect *rect);
 */
PHP_FUNCTION(SDL_SetTextInputRect)
{
	zval *z_rect;
	SDL_Rect rect;

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "O", &z_rect, get_php_sdl_rect_ce()))
	{
		return;
	}
	if (zval_to_sdl_rect(z_rect, &rect))
	{
		SDL_SetTextInputRect(&rect);
	}
}
/* }}} */

/* {{{ proto bool SDL_HasScreenKeyboardSupport(void)

 *  \brief Returns whether the platform has some screen keyboard support.
 *
 *  \return SDL_TRUE if some keyboard support is available else SDL_FALSE.
 *
 *  \note Not all screen keyboard functions are supported on all platforms.
 *
 *  \sa SDL_IsScreenKeyboardShown()
 extern DECLSPEC SDL_bool SDLCALL SDL_HasScreenKeyboardSupport(void);
 */
PHP_FUNCTION(SDL_HasScreenKeyboardSupport)
{
	if (zend_parse_parameters_none() == FAILURE)
	{
		return;
	}
	RETVAL_BOOL(SDL_HasScreenKeyboardSupport());
}
/* }}} */

/* {{{ proto bool SDL_IsScreenKeyboardShown(SDL_Window window)

 *  \brief Returns whether the screen keyboard is shown for given window.
 *
 *  \param window The window for which screen keyboard should be queried.
 *
 *  \return SDL_TRUE if screen keyboard is shown else SDL_FALSE.
 *
 *  \sa SDL_HasScreenKeyboardSupport()
 extern DECLSPEC SDL_bool SDLCALL SDL_IsScreenKeyboardShown(SDL_Window *window);
 */
PHP_FUNCTION(SDL_IsScreenKeyboardShown)
{
	zval *z_window;
	SDL_Window *window;

	if (FAILURE == zend_parse_parameters(ZEND_NUM_ARGS(), "O", &z_window, get_php_sdl_window_ce()))
	{
		return;
	}
	window = zval_to_sdl_window(z_window);
	if (window)
	{
		RETVAL_BOOL(SDL_IsScreenKeyboardShown(window));
	}
}
/* }}} */

/* {{{ MINIT */
PHP_MINIT_FUNCTION(sdl_keyboard)
{
	/* from SDL_scancode.h */

	/* enum SDL_Scancode */
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_UNKNOWN", SDL_SCANCODE_UNKNOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_A", SDL_SCANCODE_A, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_B", SDL_SCANCODE_B, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_C", SDL_SCANCODE_C, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_D", SDL_SCANCODE_D, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_E", SDL_SCANCODE_E, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F", SDL_SCANCODE_F, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_G", SDL_SCANCODE_G, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_H", SDL_SCANCODE_H, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_I", SDL_SCANCODE_I, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_J", SDL_SCANCODE_J, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_K", SDL_SCANCODE_K, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_L", SDL_SCANCODE_L, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_M", SDL_SCANCODE_M, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_N", SDL_SCANCODE_N, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_O", SDL_SCANCODE_O, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_P", SDL_SCANCODE_P, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_Q", SDL_SCANCODE_Q, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_R", SDL_SCANCODE_R, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_S", SDL_SCANCODE_S, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_T", SDL_SCANCODE_T, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_U", SDL_SCANCODE_U, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_V", SDL_SCANCODE_V, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_W", SDL_SCANCODE_W, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_X", SDL_SCANCODE_X, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_Y", SDL_SCANCODE_Y, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_Z", SDL_SCANCODE_Z, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_1", SDL_SCANCODE_1, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_2", SDL_SCANCODE_2, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_3", SDL_SCANCODE_3, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_4", SDL_SCANCODE_4, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_5", SDL_SCANCODE_5, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_6", SDL_SCANCODE_6, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_7", SDL_SCANCODE_7, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_8", SDL_SCANCODE_8, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_9", SDL_SCANCODE_9, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_0", SDL_SCANCODE_0, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_RETURN", SDL_SCANCODE_RETURN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_ESCAPE", SDL_SCANCODE_ESCAPE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_BACKSPACE", SDL_SCANCODE_BACKSPACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_TAB", SDL_SCANCODE_TAB, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_SPACE", SDL_SCANCODE_SPACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_MINUS", SDL_SCANCODE_MINUS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_EQUALS", SDL_SCANCODE_EQUALS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LEFTBRACKET", SDL_SCANCODE_LEFTBRACKET, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_RIGHTBRACKET", SDL_SCANCODE_RIGHTBRACKET, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_BACKSLASH", SDL_SCANCODE_BACKSLASH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_NONUSHASH", SDL_SCANCODE_NONUSHASH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_SEMICOLON", SDL_SCANCODE_SEMICOLON, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_APOSTROPHE", SDL_SCANCODE_APOSTROPHE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_GRAVE", SDL_SCANCODE_GRAVE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_COMMA", SDL_SCANCODE_COMMA, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_PERIOD", SDL_SCANCODE_PERIOD, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_SLASH", SDL_SCANCODE_SLASH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_CAPSLOCK", SDL_SCANCODE_CAPSLOCK, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F1", SDL_SCANCODE_F1, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F2", SDL_SCANCODE_F2, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F3", SDL_SCANCODE_F3, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F4", SDL_SCANCODE_F4, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F5", SDL_SCANCODE_F5, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F6", SDL_SCANCODE_F6, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F7", SDL_SCANCODE_F7, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F8", SDL_SCANCODE_F8, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F9", SDL_SCANCODE_F9, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F10", SDL_SCANCODE_F10, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F11", SDL_SCANCODE_F11, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F12", SDL_SCANCODE_F12, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_PRINTSCREEN", SDL_SCANCODE_PRINTSCREEN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_SCROLLLOCK", SDL_SCANCODE_SCROLLLOCK, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_PAUSE", SDL_SCANCODE_PAUSE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_INSERT", SDL_SCANCODE_INSERT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_HOME", SDL_SCANCODE_HOME, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_PAGEUP", SDL_SCANCODE_PAGEUP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_DELETE", SDL_SCANCODE_DELETE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_END", SDL_SCANCODE_END, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_PAGEDOWN", SDL_SCANCODE_PAGEDOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_RIGHT", SDL_SCANCODE_RIGHT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LEFT", SDL_SCANCODE_LEFT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_DOWN", SDL_SCANCODE_DOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_UP", SDL_SCANCODE_UP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_NUMLOCKCLEAR", SDL_SCANCODE_NUMLOCKCLEAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_DIVIDE", SDL_SCANCODE_KP_DIVIDE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_MULTIPLY", SDL_SCANCODE_KP_MULTIPLY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_MINUS", SDL_SCANCODE_KP_MINUS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_PLUS", SDL_SCANCODE_KP_PLUS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_ENTER", SDL_SCANCODE_KP_ENTER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_1", SDL_SCANCODE_KP_1, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_2", SDL_SCANCODE_KP_2, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_3", SDL_SCANCODE_KP_3, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_4", SDL_SCANCODE_KP_4, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_5", SDL_SCANCODE_KP_5, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_6", SDL_SCANCODE_KP_6, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_7", SDL_SCANCODE_KP_7, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_8", SDL_SCANCODE_KP_8, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_9", SDL_SCANCODE_KP_9, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_0", SDL_SCANCODE_KP_0, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_PERIOD", SDL_SCANCODE_KP_PERIOD, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_NONUSBACKSLASH", SDL_SCANCODE_NONUSBACKSLASH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_APPLICATION", SDL_SCANCODE_APPLICATION, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_POWER", SDL_SCANCODE_POWER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_EQUALS", SDL_SCANCODE_KP_EQUALS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F13", SDL_SCANCODE_F13, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F14", SDL_SCANCODE_F14, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F15", SDL_SCANCODE_F15, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F16", SDL_SCANCODE_F16, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F17", SDL_SCANCODE_F17, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F18", SDL_SCANCODE_F18, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F19", SDL_SCANCODE_F19, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F20", SDL_SCANCODE_F20, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F21", SDL_SCANCODE_F21, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F22", SDL_SCANCODE_F22, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F23", SDL_SCANCODE_F23, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_F24", SDL_SCANCODE_F24, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_EXECUTE", SDL_SCANCODE_EXECUTE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_HELP", SDL_SCANCODE_HELP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_MENU", SDL_SCANCODE_MENU, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_SELECT", SDL_SCANCODE_SELECT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_STOP", SDL_SCANCODE_STOP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AGAIN", SDL_SCANCODE_AGAIN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_UNDO", SDL_SCANCODE_UNDO, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_CUT", SDL_SCANCODE_CUT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_COPY", SDL_SCANCODE_COPY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_PASTE", SDL_SCANCODE_PASTE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_FIND", SDL_SCANCODE_FIND, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_MUTE", SDL_SCANCODE_MUTE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_VOLUMEUP", SDL_SCANCODE_VOLUMEUP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_VOLUMEDOWN", SDL_SCANCODE_VOLUMEDOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_COMMA", SDL_SCANCODE_KP_COMMA, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_EQUALSAS400", SDL_SCANCODE_KP_EQUALSAS400, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_INTERNATIONAL1", SDL_SCANCODE_INTERNATIONAL1, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_INTERNATIONAL2", SDL_SCANCODE_INTERNATIONAL2, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_INTERNATIONAL3", SDL_SCANCODE_INTERNATIONAL3, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_INTERNATIONAL4", SDL_SCANCODE_INTERNATIONAL4, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_INTERNATIONAL5", SDL_SCANCODE_INTERNATIONAL5, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_INTERNATIONAL6", SDL_SCANCODE_INTERNATIONAL6, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_INTERNATIONAL7", SDL_SCANCODE_INTERNATIONAL7, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_INTERNATIONAL8", SDL_SCANCODE_INTERNATIONAL8, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_INTERNATIONAL9", SDL_SCANCODE_INTERNATIONAL9, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LANG1", SDL_SCANCODE_LANG1, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LANG2", SDL_SCANCODE_LANG2, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LANG3", SDL_SCANCODE_LANG3, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LANG4", SDL_SCANCODE_LANG4, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LANG5", SDL_SCANCODE_LANG5, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LANG6", SDL_SCANCODE_LANG6, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LANG7", SDL_SCANCODE_LANG7, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LANG8", SDL_SCANCODE_LANG8, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LANG9", SDL_SCANCODE_LANG9, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_ALTERASE", SDL_SCANCODE_ALTERASE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_SYSREQ", SDL_SCANCODE_SYSREQ, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_CANCEL", SDL_SCANCODE_CANCEL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_CLEAR", SDL_SCANCODE_CLEAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_PRIOR", SDL_SCANCODE_PRIOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_RETURN2", SDL_SCANCODE_RETURN2, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_SEPARATOR", SDL_SCANCODE_SEPARATOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_OUT", SDL_SCANCODE_OUT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_OPER", SDL_SCANCODE_OPER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_CLEARAGAIN", SDL_SCANCODE_CLEARAGAIN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_CRSEL", SDL_SCANCODE_CRSEL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_EXSEL", SDL_SCANCODE_EXSEL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_00", SDL_SCANCODE_KP_00, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_000", SDL_SCANCODE_KP_000, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_THOUSANDSSEPARATOR", SDL_SCANCODE_THOUSANDSSEPARATOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_DECIMALSEPARATOR", SDL_SCANCODE_DECIMALSEPARATOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_CURRENCYUNIT", SDL_SCANCODE_CURRENCYUNIT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_CURRENCYSUBUNIT", SDL_SCANCODE_CURRENCYSUBUNIT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_LEFTPAREN", SDL_SCANCODE_KP_LEFTPAREN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_RIGHTPAREN", SDL_SCANCODE_KP_RIGHTPAREN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_LEFTBRACE", SDL_SCANCODE_KP_LEFTBRACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_RIGHTBRACE", SDL_SCANCODE_KP_RIGHTBRACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_TAB", SDL_SCANCODE_KP_TAB, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_BACKSPACE", SDL_SCANCODE_KP_BACKSPACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_A", SDL_SCANCODE_KP_A, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_B", SDL_SCANCODE_KP_B, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_C", SDL_SCANCODE_KP_C, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_D", SDL_SCANCODE_KP_D, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_E", SDL_SCANCODE_KP_E, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_F", SDL_SCANCODE_KP_F, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_XOR", SDL_SCANCODE_KP_XOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_POWER", SDL_SCANCODE_KP_POWER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_PERCENT", SDL_SCANCODE_KP_PERCENT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_LESS", SDL_SCANCODE_KP_LESS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_GREATER", SDL_SCANCODE_KP_GREATER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_AMPERSAND", SDL_SCANCODE_KP_AMPERSAND, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_DBLAMPERSAND", SDL_SCANCODE_KP_DBLAMPERSAND, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_VERTICALBAR", SDL_SCANCODE_KP_VERTICALBAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_DBLVERTICALBAR", SDL_SCANCODE_KP_DBLVERTICALBAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_COLON", SDL_SCANCODE_KP_COLON, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_HASH", SDL_SCANCODE_KP_HASH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_SPACE", SDL_SCANCODE_KP_SPACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_AT", SDL_SCANCODE_KP_AT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_EXCLAM", SDL_SCANCODE_KP_EXCLAM, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_MEMSTORE", SDL_SCANCODE_KP_MEMSTORE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_MEMRECALL", SDL_SCANCODE_KP_MEMRECALL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_MEMCLEAR", SDL_SCANCODE_KP_MEMCLEAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_MEMADD", SDL_SCANCODE_KP_MEMADD, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_MEMSUBTRACT", SDL_SCANCODE_KP_MEMSUBTRACT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_MEMMULTIPLY", SDL_SCANCODE_KP_MEMMULTIPLY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_MEMDIVIDE", SDL_SCANCODE_KP_MEMDIVIDE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_PLUSMINUS", SDL_SCANCODE_KP_PLUSMINUS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_CLEAR", SDL_SCANCODE_KP_CLEAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_CLEARENTRY", SDL_SCANCODE_KP_CLEARENTRY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_BINARY", SDL_SCANCODE_KP_BINARY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_OCTAL", SDL_SCANCODE_KP_OCTAL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_DECIMAL", SDL_SCANCODE_KP_DECIMAL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KP_HEXADECIMAL", SDL_SCANCODE_KP_HEXADECIMAL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LCTRL", SDL_SCANCODE_LCTRL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LSHIFT", SDL_SCANCODE_LSHIFT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LALT", SDL_SCANCODE_LALT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_LGUI", SDL_SCANCODE_LGUI, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_RCTRL", SDL_SCANCODE_RCTRL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_RSHIFT", SDL_SCANCODE_RSHIFT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_RALT", SDL_SCANCODE_RALT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_RGUI", SDL_SCANCODE_RGUI, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_MODE", SDL_SCANCODE_MODE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AUDIONEXT", SDL_SCANCODE_AUDIONEXT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AUDIOPREV", SDL_SCANCODE_AUDIOPREV, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AUDIOSTOP", SDL_SCANCODE_AUDIOSTOP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AUDIOPLAY", SDL_SCANCODE_AUDIOPLAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AUDIOMUTE", SDL_SCANCODE_AUDIOMUTE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_MEDIASELECT", SDL_SCANCODE_MEDIASELECT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_WWW", SDL_SCANCODE_WWW, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_MAIL", SDL_SCANCODE_MAIL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_CALCULATOR", SDL_SCANCODE_CALCULATOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_COMPUTER", SDL_SCANCODE_COMPUTER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AC_SEARCH", SDL_SCANCODE_AC_SEARCH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AC_HOME", SDL_SCANCODE_AC_HOME, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AC_BACK", SDL_SCANCODE_AC_BACK, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AC_FORWARD", SDL_SCANCODE_AC_FORWARD, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AC_STOP", SDL_SCANCODE_AC_STOP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AC_REFRESH", SDL_SCANCODE_AC_REFRESH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_AC_BOOKMARKS", SDL_SCANCODE_AC_BOOKMARKS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_BRIGHTNESSDOWN", SDL_SCANCODE_BRIGHTNESSDOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_BRIGHTNESSUP", SDL_SCANCODE_BRIGHTNESSUP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_DISPLAYSWITCH", SDL_SCANCODE_DISPLAYSWITCH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KBDILLUMTOGGLE", SDL_SCANCODE_KBDILLUMTOGGLE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KBDILLUMDOWN", SDL_SCANCODE_KBDILLUMDOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_KBDILLUMUP", SDL_SCANCODE_KBDILLUMUP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_EJECT", SDL_SCANCODE_EJECT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_SLEEP", SDL_SCANCODE_SLEEP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_APP1", SDL_SCANCODE_APP1, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_SCANCODE_APP2", SDL_SCANCODE_APP2, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDL_NUM_SCANCODES", SDL_NUM_SCANCODES, CONST_CS | CONST_PERSISTENT);

	/* From SDL_keycode.h */

	REGISTER_LONG_CONSTANT("SDLK_UNKNOWN", SDLK_UNKNOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_RETURN", SDLK_RETURN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_ESCAPE", SDLK_ESCAPE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_BACKSPACE", SDLK_BACKSPACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_TAB", SDLK_TAB, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_SPACE", SDLK_SPACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_EXCLAIM", SDLK_EXCLAIM, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_QUOTEDBL", SDLK_QUOTEDBL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_HASH", SDLK_HASH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_PERCENT", SDLK_PERCENT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_DOLLAR", SDLK_DOLLAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AMPERSAND", SDLK_AMPERSAND, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_QUOTE", SDLK_QUOTE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_LEFTPAREN", SDLK_LEFTPAREN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_RIGHTPAREN", SDLK_RIGHTPAREN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_ASTERISK", SDLK_ASTERISK, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_PLUS", SDLK_PLUS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_COMMA", SDLK_COMMA, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_MINUS", SDLK_MINUS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_PERIOD", SDLK_PERIOD, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_SLASH", SDLK_SLASH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_0", SDLK_0, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_1", SDLK_1, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_2", SDLK_2, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_3", SDLK_3, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_4", SDLK_4, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_5", SDLK_5, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_6", SDLK_6, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_7", SDLK_7, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_8", SDLK_8, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_9", SDLK_9, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_COLON", SDLK_COLON, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_SEMICOLON", SDLK_SEMICOLON, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_LESS", SDLK_LESS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_EQUALS", SDLK_EQUALS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_GREATER", SDLK_GREATER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_QUESTION", SDLK_QUESTION, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AT", SDLK_AT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_LEFTBRACKET", SDLK_LEFTBRACKET, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_BACKSLASH", SDLK_BACKSLASH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_RIGHTBRACKET", SDLK_RIGHTBRACKET, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_CARET", SDLK_CARET, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_UNDERSCORE", SDLK_UNDERSCORE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_BACKQUOTE", SDLK_BACKQUOTE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_a", SDLK_a, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_b", SDLK_b, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_c", SDLK_c, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_d", SDLK_d, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_e", SDLK_e, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_f", SDLK_f, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_g", SDLK_g, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_h", SDLK_h, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_i", SDLK_i, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_j", SDLK_j, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_k", SDLK_k, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_l", SDLK_l, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_m", SDLK_m, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_n", SDLK_n, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_o", SDLK_o, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_p", SDLK_p, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_q", SDLK_q, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_r", SDLK_r, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_s", SDLK_s, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_t", SDLK_t, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_u", SDLK_u, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_v", SDLK_v, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_w", SDLK_w, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_x", SDLK_x, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_y", SDLK_y, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_z", SDLK_z, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_CAPSLOCK", SDLK_CAPSLOCK, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F1", SDLK_F1, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F2", SDLK_F2, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F3", SDLK_F3, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F4", SDLK_F4, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F5", SDLK_F5, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F6", SDLK_F6, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F7", SDLK_F7, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F8", SDLK_F8, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F9", SDLK_F9, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F10", SDLK_F10, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F11", SDLK_F11, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F12", SDLK_F12, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_PRINTSCREEN", SDLK_PRINTSCREEN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_SCROLLLOCK", SDLK_SCROLLLOCK, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_PAUSE", SDLK_PAUSE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_INSERT", SDLK_INSERT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_HOME", SDLK_HOME, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_PAGEUP", SDLK_PAGEUP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_DELETE", SDLK_DELETE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_END", SDLK_END, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_PAGEDOWN", SDLK_PAGEDOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_RIGHT", SDLK_RIGHT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_LEFT", SDLK_LEFT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_DOWN", SDLK_DOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_UP", SDLK_UP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_NUMLOCKCLEAR", SDLK_NUMLOCKCLEAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_DIVIDE", SDLK_KP_DIVIDE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_MULTIPLY", SDLK_KP_MULTIPLY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_MINUS", SDLK_KP_MINUS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_PLUS", SDLK_KP_PLUS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_ENTER", SDLK_KP_ENTER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_1", SDLK_KP_1, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_2", SDLK_KP_2, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_3", SDLK_KP_3, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_4", SDLK_KP_4, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_5", SDLK_KP_5, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_6", SDLK_KP_6, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_7", SDLK_KP_7, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_8", SDLK_KP_8, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_9", SDLK_KP_9, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_0", SDLK_KP_0, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_PERIOD", SDLK_KP_PERIOD, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_APPLICATION", SDLK_APPLICATION, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_POWER", SDLK_POWER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_EQUALS", SDLK_KP_EQUALS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F13", SDLK_F13, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F14", SDLK_F14, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F15", SDLK_F15, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F16", SDLK_F16, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F17", SDLK_F17, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F18", SDLK_F18, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F19", SDLK_F19, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F20", SDLK_F20, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F21", SDLK_F21, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F22", SDLK_F22, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F23", SDLK_F23, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_F24", SDLK_F24, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_EXECUTE", SDLK_EXECUTE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_HELP", SDLK_HELP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_MENU", SDLK_MENU, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_SELECT", SDLK_SELECT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_STOP", SDLK_STOP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AGAIN", SDLK_AGAIN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_UNDO", SDLK_UNDO, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_CUT", SDLK_CUT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_COPY", SDLK_COPY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_PASTE", SDLK_PASTE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_FIND", SDLK_FIND, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_MUTE", SDLK_MUTE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_VOLUMEUP", SDLK_VOLUMEUP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_VOLUMEDOWN", SDLK_VOLUMEDOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_COMMA", SDLK_KP_COMMA, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_EQUALSAS400", SDLK_KP_EQUALSAS400, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_ALTERASE", SDLK_ALTERASE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_SYSREQ", SDLK_SYSREQ, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_CANCEL", SDLK_CANCEL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_CLEAR", SDLK_CLEAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_PRIOR", SDLK_PRIOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_RETURN2", SDLK_RETURN2, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_SEPARATOR", SDLK_SEPARATOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_OUT", SDLK_OUT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_OPER", SDLK_OPER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_CLEARAGAIN", SDLK_CLEARAGAIN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_CRSEL", SDLK_CRSEL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_EXSEL", SDLK_EXSEL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_00", SDLK_KP_00, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_000", SDLK_KP_000, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_THOUSANDSSEPARATOR", SDLK_THOUSANDSSEPARATOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_DECIMALSEPARATOR", SDLK_DECIMALSEPARATOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_CURRENCYUNIT", SDLK_CURRENCYUNIT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_CURRENCYSUBUNIT", SDLK_CURRENCYSUBUNIT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_LEFTPAREN", SDLK_KP_LEFTPAREN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_RIGHTPAREN", SDLK_KP_RIGHTPAREN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_LEFTBRACE", SDLK_KP_LEFTBRACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_RIGHTBRACE", SDLK_KP_RIGHTBRACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_TAB", SDLK_KP_TAB, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_BACKSPACE", SDLK_KP_BACKSPACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_A", SDLK_KP_A, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_B", SDLK_KP_B, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_C", SDLK_KP_C, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_D", SDLK_KP_D, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_E", SDLK_KP_E, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_F", SDLK_KP_F, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_XOR", SDLK_KP_XOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_POWER", SDLK_KP_POWER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_PERCENT", SDLK_KP_PERCENT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_LESS", SDLK_KP_LESS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_GREATER", SDLK_KP_GREATER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_AMPERSAND", SDLK_KP_AMPERSAND, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_DBLAMPERSAND", SDLK_KP_DBLAMPERSAND, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_VERTICALBAR", SDLK_KP_VERTICALBAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_DBLVERTICALBAR", SDLK_KP_DBLVERTICALBAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_COLON", SDLK_KP_COLON, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_HASH", SDLK_KP_HASH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_SPACE", SDLK_KP_SPACE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_AT", SDLK_KP_AT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_EXCLAM", SDLK_KP_EXCLAM, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_MEMSTORE", SDLK_KP_MEMSTORE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_MEMRECALL", SDLK_KP_MEMRECALL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_MEMCLEAR", SDLK_KP_MEMCLEAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_MEMADD", SDLK_KP_MEMADD, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_MEMSUBTRACT", SDLK_KP_MEMSUBTRACT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_MEMMULTIPLY", SDLK_KP_MEMMULTIPLY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_MEMDIVIDE", SDLK_KP_MEMDIVIDE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_PLUSMINUS", SDLK_KP_PLUSMINUS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_CLEAR", SDLK_KP_CLEAR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_CLEARENTRY", SDLK_KP_CLEARENTRY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_BINARY", SDLK_KP_BINARY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_OCTAL", SDLK_KP_OCTAL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_DECIMAL", SDLK_KP_DECIMAL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KP_HEXADECIMAL", SDLK_KP_HEXADECIMAL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_LCTRL", SDLK_LCTRL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_LSHIFT", SDLK_LSHIFT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_LALT", SDLK_LALT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_LGUI", SDLK_LGUI, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_RCTRL", SDLK_RCTRL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_RSHIFT", SDLK_RSHIFT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_RALT", SDLK_RALT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_RGUI", SDLK_RGUI, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_MODE", SDLK_MODE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AUDIONEXT", SDLK_AUDIONEXT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AUDIOPREV", SDLK_AUDIOPREV, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AUDIOSTOP", SDLK_AUDIOSTOP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AUDIOPLAY", SDLK_AUDIOPLAY, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AUDIOMUTE", SDLK_AUDIOMUTE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_MEDIASELECT", SDLK_MEDIASELECT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_WWW", SDLK_WWW, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_MAIL", SDLK_MAIL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_CALCULATOR", SDLK_CALCULATOR, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_COMPUTER", SDLK_COMPUTER, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AC_SEARCH", SDLK_AC_SEARCH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AC_HOME", SDLK_AC_HOME, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AC_BACK", SDLK_AC_BACK, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AC_FORWARD", SDLK_AC_FORWARD, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AC_STOP", SDLK_AC_STOP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AC_REFRESH", SDLK_AC_REFRESH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_AC_BOOKMARKS", SDLK_AC_BOOKMARKS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_BRIGHTNESSDOWN", SDLK_BRIGHTNESSDOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_BRIGHTNESSUP", SDLK_BRIGHTNESSUP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_DISPLAYSWITCH", SDLK_DISPLAYSWITCH, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KBDILLUMTOGGLE", SDLK_KBDILLUMTOGGLE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KBDILLUMDOWN", SDLK_KBDILLUMDOWN, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_KBDILLUMUP", SDLK_KBDILLUMUP, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_EJECT", SDLK_EJECT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("SDLK_SLEEP", SDLK_SLEEP, CONST_CS | CONST_PERSISTENT);

	/* enum SDL_Keymod */
	REGISTER_LONG_CONSTANT("KMOD_NONE", KMOD_NONE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_LSHIFT", KMOD_LSHIFT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_RSHIFT", KMOD_RSHIFT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_LCTRL", KMOD_LCTRL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_RCTRL", KMOD_RCTRL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_LALT", KMOD_LALT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_RALT", KMOD_RALT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_LGUI", KMOD_LGUI, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_RGUI", KMOD_RGUI, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_NUM", KMOD_NUM, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_CAPS", KMOD_CAPS, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_MODE", KMOD_MODE, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_RESERVED", KMOD_RESERVED, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_CTRL", KMOD_CTRL, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_SHIFT", KMOD_SHIFT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_ALT", KMOD_ALT, CONST_CS | CONST_PERSISTENT);
	REGISTER_LONG_CONSTANT("KMOD_GUI", KMOD_GUI, CONST_CS | CONST_PERSISTENT);

	return SUCCESS;
}
/* }}} */
